
/*
 *	config.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1995 by Robert Krten, all rights reserved.
 *
 *	This module contains the configuration reader.  It reads,
 *	by default, /etc/knews, and parses the various options.
 *
 *	1995 12 20	R. Krten		created
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include "newslib.h"

static	int		doneInit = 0;
static	ConfigT	*masterConfigList;
static	int		nItems;

char	*configFname = NULL;
extern	char *progname;

ConfigT	*
getConfig (item)
char	*item;
{
	int		i;

	if (!doneInit) {
		configInit ();
	}
	for (i = 0; i < nItems; i++) {
		if (!strcmp (masterConfigList [i].items [0].string, item)) {
			return (&masterConfigList [i]);
		}
	}
	return (NULL);
}

void
configInit ()
{
	FILE	*fp;
	char	*cfile;
	char	buf [BUFSIZ], word [BUFSIZ];
	char	*ptr;
	int		i;

	cfile = (configFname == NULL) ? "/etc/knews" : configFname;

	if ((fp = fopen (cfile, "r")) == NULL) {
		fprintf (stderr, "%s:  can't open configuration file %s for r\n", progname, cfile);
		perror (NULL);
		exit (1);
	}

	// see how many keyword lines there are in the file

	nItems = 0;
	while (!feof (fp) && fgets (buf, BUFSIZ, fp) != NULL) {
		ptr = buf;
		if (getWord (&ptr, word) && *word != '#') {
			nItems++;
		}
	}
	
	rewind (fp);

	// allocate room for the keywords

	if ((masterConfigList = calloc (nItems, sizeof (ConfigT))) == NULL) {
		fprintf (stderr, "%s:  can't allocate %d bytes\n", progname, nItems * sizeof (ConfigT));
		perror (NULL);
		exit (1);
	}

 	// and read them in

	nItems = 0;
	while (!feof (fp) && fgets (buf, BUFSIZ, fp) != NULL) {
		ptr = buf;
		if (getWord (&ptr, word) && *word != '#') {
			i = 0;
			do {
//DEBUG			printf ("ENTRY [%d] item [%d] is %s\n", nItems, i, word);
				masterConfigList [nItems].items [i].string = strdup (word);
				masterConfigList [nItems].items [i].number = strtol (word, NULL, 0);
				i++;
			} while (getWord (&ptr, word));
			masterConfigList [nItems].nitems = i;
			nItems++;
		}
	}

	// all done, close file and set flag

	fclose (fp);
	doneInit = 1;
}

