
/*
 *	getWord.c
 *
 *	(C) Copyright 1992 by Robert Krten, all rights reserved.
 *
 *	This library module contains the following routine(s):
 *
 *		getWord
 *
 *	getWord is a general purpose text function that will separate out a word
 *		from a string.  It will advance the passed pointer to point to just
 *		after the end of the current word.
 *
 *	---------------------------------------------------------------------------
 *
 *	1992 ?? ??	R. Krten		created
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "newslib.h"
#include "getWord.h"

int
getWord (
	char	**ptr,				/* buffer pointer, advanced */
	char	*word				/* buffer to put word in */
) {
	int		howMany;			/* flag indicating how many chars we got */

	/*
	 *	Reset howMany flag, we didn't get any characters yet
	*/

	howMany = 0;

	/*
	 *	Skip initial spaces, halt if EOS
	*/

	while (**ptr && isspace (**ptr)) {
		++*ptr;
	}

	/*
	 *	Accumulate non-space characters, halt if EOS
	*/

	while (**ptr && !isspace (**ptr)) {
		*word++ = **ptr;
		++*ptr;
		howMany++;
	}

	/*
	 *	Terminate existing word (if none yet, it will yield empty string)
	*/

	*word = 0;

	/*
	 *	Skip trailing spaces, halt if EOS
	*/

	while (**ptr && isspace (**ptr)) {
		++*ptr;
	}
	/*
	 *	Return the number of characters that we got.
	*/

	return (howMany);
}

