
/*
 *	ipc.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1995 by Robert Krten, all rights reserved.
 *
 *	This module contains all of the IPC server messages.
 *
 *	1995 12 30	R. Krten		stole from CLID IPC (1993 02 25)
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include "newslib.h"

extern	char	*progname;					/* main.c */
extern	char	*version;					/* version.c */

extern	int		optv;						/* main.c */

/*
 *	Static Data Areas
*/

static	int		KNEWS_TID = 0;
static	MsgT	KNEWS_IPCData;

/*
 *	IPCEraseDatabase ()
*/

int
IPCEraseDatabase ()
{
	checkAttach ();
	KNEWS_IPCData.erasedatabase.type = KNewsMsg_EraseDatabase;
	KNEWS_IPC ();
	return (KNEWS_IPCData.erasedatabase_reply.status);
}

/*
 *	IPCAddNewsgroup (group)
*/

int
IPCAddNewsgroup (group)
char	*group;
{
	checkAttach ();
	KNEWS_IPCData.addnewsgroup.type = KNewsMsg_AddNewsgroup;
	strcpy (KNEWS_IPCData.addnewsgroup.group, group);
	KNEWS_IPC ();
	return (KNEWS_IPCData.addnewsgroup_reply.status);
}

/*
 *	IPCAddArticle
*/

int
IPCAddArticle (group, artnum, expiry, offset, length)
char	*group;
U32		artnum;
U32		expiry;
U32		offset;
U32		length;
{
	checkAttach ();
	KNEWS_IPCData.addarticle.type = KNewsMsg_AddArticle;
	strcpy (KNEWS_IPCData.addarticle.group, group);
	KNEWS_IPCData.addarticle.artnum = artnum;
	KNEWS_IPCData.addarticle.expiry = expiry;
	KNEWS_IPCData.addarticle.offset = offset;
	KNEWS_IPCData.addarticle.length = length;
	KNEWS_IPC ();
	return (KNEWS_IPCData.addarticle_reply.status);
}

/*
 *	IPCExpire
*/

int
IPCExpire (t)
time_t	t;
{
	checkAttach ();
	KNEWS_IPCData.expire.type = KNewsMsg_Expire;
	KNEWS_IPCData.expire.when = t;
	KNEWS_IPC ();
	return (KNEWS_IPCData.expire_reply.status);
}

/*
 *	IPCWriteDebug
*/

int
IPCWriteDebug ()
{
	checkAttach ();
	KNEWS_IPCData.writedebug.type = KNewsMsg_WriteDebug;
	KNEWS_IPC ();
	return (KNEWS_IPCData.writedebug_reply.status);
}

/*
 *	PRIVATE FUNCTIONS
*/

/*
 *	KNEWS_IPC (IPC message number)
 *
 *	This routine will call the server with the global buffer KNEWS_IPCData,
 *	and will stuff in the message number passed as the argument.
 *
 *	Should the server not exist, this routine will stuff the .returnValue
 *	field with KNEWS_NoServer.  Otherwise, no fields are affected.
*/

void
KNEWS_IPC ()
{
	if (KNEWS_TID == 0 || KNEWS_TID == -1) {
		errno = KNEWS_IPCData.status = ESRCH;
		return;
	}
	if (Send (KNEWS_TID, &KNEWS_IPCData, &KNEWS_IPCData, sizeof (KNEWS_IPCData), sizeof (KNEWS_IPCData))) {
		errno = KNEWS_IPCData.status = ESRCH;
	} else {
		errno = KNEWS_IPCData.status;
	}
}

void
checkAttach ()
{
	static	char	KNEWS_serverName [BUFSIZ];

	if (!KNEWS_TID) {
		sprintf (KNEWS_serverName, "/PARSE/VFsys.knews");
		KNEWS_TID = qnx_name_locate (0, KNEWS_serverName, sizeof (MsgT), NULL);
	}
}

