
/*
 *	ipc.h
 *
 *	QNX 4
 *
 *	(C) Copyright 1995, 1996 by Robert Krten, all rights reserved.
 *
 *	This module contains the definitions for the IPC messages that
 *	are sent between various clients and the KNewsVFsys server.
 *
 *	1995 12 30	R. Krten		created
 *	1996 01 06	R. Krten		modified for new VFsys-style
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include <sys/sys_msg.h>
#include <sys/io_msg.h>
#include "../lib/portable.h"

#define	MinKNewsMsg					0x8000
#define	KNewsMsg_AddNewsgroup		(MinKNewsMsg + 0)
#define	KNewsMsg_AddArticle			(MinKNewsMsg + 1)
#define	KNewsMsg_Expire				(MinKNewsMsg + 2)
#define	KNewsMsg_EraseDatabase		(MinKNewsMsg + 3)
#define	KNewsMsg_WriteDebug			(MinKNewsMsg + 4)
#define	MaxKNewsMsg					(MinKNewsMsg + 4)

#define	KNewsMaxNewsgroupLen		128

#define	MAXMSGSIZE					65536

typedef struct
{
	msg_t           type;
	char			group [KNewsMaxNewsgroupLen + 1];
}	KNEWSAddNewsgroupT;

typedef struct
{
	msg_t           status;
}	KNEWSAddNewsgroupRT;

typedef struct
{
	msg_t           type;
	char			group [KNewsMaxNewsgroupLen + 1];
	U32				artnum;
	U32				expiry;
	U32				offset;
	U32				length;
}	KNEWSAddArticleT;

typedef struct
{
	msg_t           status;
}	KNEWSAddArticleRT;

typedef struct
{
	msg_t           type;
	time_t			when;
}	KNEWSExpireT;

typedef struct
{
	msg_t           status;
}	KNEWSExpireRT;

typedef struct
{
	msg_t           type;
}	KNEWSEraseDatabaseT;

typedef struct
{
	msg_t           status;
}	KNEWSEraseDatabaseRT;

typedef struct
{
	msg_t           type;
}	KNEWSWriteDebugT;

typedef struct
{
	msg_t           status;
}	KNEWSWriteDebugRT;

/*
 *	io messages
*/

typedef union
{
	short unsigned					type;
	short unsigned					status;
	struct _sysmsg_hdr				sysmsg;
	struct _sysmsg_hdr_reply		sysmsg_reply;
	struct _io_open					open;			/* IO_HANDLE uses this one also.	*/
	struct _io_open_reply			open_reply;
	struct _io_dup					dup;			/* Used for fork, exec, dup, dup2()'s. */
	struct _io_dup_reply			dup_reply;
	struct _io_close				close;
	struct _io_close_reply			close_reply;
	struct _io_read					read;
	struct _io_read_reply			read_reply;
	struct _io_write				write;
	struct _io_write_reply			write_reply;
	struct _io_qioctl				ioctl;
	struct _io_qioctl_reply			ioctl_reply;
	struct _io_utime				utime;
	struct _io_utime_reply			utime_reply;
	struct _io_chmod				chmod;
	struct _io_chmod_reply   		chmod_reply;
	struct _io_chown				chown;
	struct _io_chown_reply			chown_reply;
	struct _io_fstat				fstat;
	struct _io_fstat_reply			fstat_reply;
	struct _io_readdir				readdir;
	struct _io_readdir_reply		readdir_reply;
	struct _io_rewinddir			rewinddir;
	struct _io_rewinddir_reply		rewinddir_reply;	
	struct _io_open					chdir;
	struct _io_open_reply			chdir_reply;
	struct _io_lseek				lseek;
	struct _io_lseek_reply			lseek_reply;

	/* user defined messages after this point */
	KNEWSAddNewsgroupT				addnewsgroup;
	KNEWSAddNewsgroupRT				addnewsgroup_reply;
	KNEWSAddArticleT				addarticle;
	KNEWSAddArticleRT				addarticle_reply;
	KNEWSExpireT					expire;
	KNEWSExpireRT					expire_reply;
	KNEWSEraseDatabaseT				erasedatabase;
	KNEWSEraseDatabaseRT			erasedatabase_reply;
	KNEWSWriteDebugT				writedebug;
	KNEWSWriteDebugRT				writedebug_reply;

	/* filler so that we have a "big enough" buffer */
	char							filler [MAXMSGSIZE];
}	MsgT;

// prototypes

int			IPCEraseDatabase (void);
int			IPCAddNewsgroup (char *);
int			IPCRemoveNewsgroup (char *);
int			IPCAddArticle (char *, U32, U32, U32, U32);
int			IPCExpire (time_t);
int			IPCWriteDebug (void);
int			IPCMinMaxArticles (char *, U32 *, U32 *);

