
/*
 *	lptr.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1995 by Robert Krten, all rights reserved.
 *
 *	This module contains lptr support routines.
 *
 *	1995 12 21	R. Krten		created
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include "newslib.h"

#define	LPTRAllocationChunk		4096		/* allocate by 4k line chunks */

extern	char	*progname;					/* main.c */
extern	char	*version;					/* version.c */
extern	int		optv;						/* main.c */

LPTRT	*lptr;								/* PUBLIC:  current LPTR object */

void
lptrReader (buf)
char	*buf;
{
	lptr -> object [lptr -> nlptrs++] = strdup (buf);
	if (lptr -> nlptrs == lptr -> nalloc) {
		lptrGrow ();
	}
	if (optv && !(lptr -> nlptrs % 100)) {
		printf ("%d\r", lptr -> nlptrs);
		fflush (stdout);
	}
}

void
lptrGrow ()
{
	int		i;

	lptr -> nalloc += LPTRAllocationChunk;
	lptr -> object = realloc (lptr -> object, sizeof (char *) * lptr -> nalloc);
	if (lptr -> object == NULL) {
		fprintf (stderr, "%s:  can't realloc lptr -> object, size requested %d bytes\n", progname, sizeof (char *) * lptr -> nalloc);
		perror (NULL);
		exit (1);
	}
	for (i = 0; i < LPTRAllocationChunk; i++) {
		lptr -> object [i + lptr -> nalloc - LPTRAllocationChunk] = NULL;
	}
}

void
lptrInit (which)
int		which;
{
	switch (which) {
	case	's':		/* starter */
		lptrInitStarter ();
		break;
	case	'e':		/* ender */
		lptrInitEnder ();
		break;
	default:
		fprintf (stderr, "%s:  error, unknown case 0x%02X in lptrInit!\n", progname, which);
		exit (1);
	}
}

void
lptrInitStarter ()
{
	if ((lptr -> object = calloc (sizeof (char *), LPTRAllocationChunk)) == NULL) {
		fprintf (stderr, "%s:  out of memory on %d bytes!\n", progname, sizeof (char *) * LPTRAllocationChunk);
		perror (NULL);
		exit (1);
	}
	lptr -> nlptrs = 0;
	lptr -> nalloc = LPTRAllocationChunk;
	if (optv) {
		printf ("LPTR Reading\n");
	}
}

void
lptrInitEnder ()
{
	int		i;

	for (i = 0; i < lptr -> nlptrs; i++) {
		if (lptr -> object [i] != NULL) {
			free (lptr -> object [i]);
		}
	}
	free (lptr -> object);
	if (optv) {
		printf ("Read %d objects\n", lptr -> nlptrs);
	}
	lptr -> nlptrs = 0;
}

int
lptrNumEntries ()
{
	return (lptr -> nlptrs);
}

int
lptrQSort (a, b)
char	**a, **b;
{
	return (strcmp (*a, *b));
}

void
lptrSort ()
{
	qsort (lptr -> object, lptr -> nlptrs, sizeof (char *), lptrQSort);
}

void
lptrWriteFP (fp)
FILE	*fp;
{
	int		i;

	for (i = 0; i < lptr -> nlptrs; i++) {
		fprintf (fp, "%s\n", lptr -> object [i]);
	}
}

void
lptrReadFP (fp)
FILE	*fp;
{
	char	buf [BUFSIZ];

	while (!feof (fp) && fgets (buf, BUFSIZ, fp) != NULL) {
		buf [strlen (buf) - 1] = 0;
		lptrReader (buf);
	}
}

