
/*
 *	This code is stolen wholesale from the NNTP distribution.
 *	I've just removed some of the extra UNIX cruft from it.
 *	I don't claim any ownership of this code... :-)
 *
 *	Certainly, if you can make it faster, by all means!
*/

extern int optv;

#include <unix.h>
#include <unistd.h>

#define SIGRET void	/* Newfangled signal() returns void, old returns int */
#define FAIL		(-1)		/* routine failed */
#define	NOHOST		(FAIL-1)	/* no such host */

/*
** Erik E. Fair <fair@ucbarpa.berkeley.edu>
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <ctype.h>
#include <stdio.h>
#include <netdb.h>
#include <sys/dir.h>
#include <time.h>
#include <stdlib.h>
#include <errno.h>
#include <setjmp.h>
#include <signal.h>
#include <string.h>

#include "newslib.h"

extern	int	errno;
extern	char	*Pname;
extern	char	*inet_ntoa();
extern	u_long	inet_addr();

/*
** Take the name of an internet host in ASCII (this may either be its
** official host name or internet number (with or without enclosing
** backets [])), and return a list of internet addresses.
**
** returns NULL for failure to find the host name in the local database,
** or for a bad internet address spec.
*/
u_long **
name_to_address(host)
char	*host;
{
	static	u_long	*host_addresses[2];
	static	u_long	haddr;

	if (host == (char *)NULL) {
		return((u_long **)NULL);
	}

	host_addresses[0] = &haddr;
	host_addresses[1] = (u_long *)NULL;

	/*
	** Is this an ASCII internet address? (either of [10.0.0.78] or
	** 10.0.0.78). We get away with the second test because hostnames
	** and domain labels are not allowed to begin in numbers.
	** (cf. RFC952, RFC882).
	*/
	if (*host == '[' || isdigit(*host)) {
		char	namebuf[128];
		register char	*cp = namebuf;

		/*
		** strip brackets [] or anything else we don't want.
		*/
		while(*host != '\0' && cp < &namebuf[sizeof(namebuf)]) {
			if (isdigit(*host) || *host == '.') {
				*cp++ = *host++;	/* copy */
			} else {
				host++;			/* skip */
			}
		}
		*cp = '\0';
		haddr = inet_addr(namebuf);
		return(&host_addresses[0]);
	} else {
		struct hostent	*hstp = gethostbyname(host);

		if (hstp == NULL) {
			return((u_long **)NULL);	/* no such host */
		}

		if (hstp->h_length != sizeof(u_long)) {
			abort();	/* this is fundamental */
		}
		return((u_long **)hstp->h_addr_list);
	}
}

/*
** given a host name (either name or internet address) and service name
** (or port number) (both in ASCII), give us a TCP connection to the
** requested service at the requested host (or give us FAIL).
*/

get_tcp_conn(host)
char	*host;
{
	register int	sock;
	u_long	**addrlist;
	struct sockaddr_in	sadr;

	if ((addrlist = name_to_address(host)) == (u_long **)NULL) {
		return(NOHOST);
	}

	sadr.sin_family = (u_short)AF_INET;	/* Only internet for now */
	sadr.sin_port = htons (119);

	for(; *addrlist != (u_long *)NULL; addrlist++) {
		bcopy((caddr_t)*addrlist, (caddr_t)&sadr.sin_addr,
			sizeof(sadr.sin_addr));

		if ((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
			return(FAIL);
		}

		if (connect(sock, (struct sockaddr *)&sadr, sizeof(sadr)) < 0) {
			int	e_save = errno;

			fprintf(stderr, "%s: %s [%s]: errno %d\n", Pname, host,
				inet_ntoa(sadr.sin_addr), errno);
			(void) close(sock);	/* dump descriptor */
			errno = e_save;
		} else {
			return(sock);
		}
	}
	return(FAIL);
}

/*
 *	Brian Kantor, UCSD 1986
 * (some bug fixes by ambar@athena.mit.edu)
 * Modified to use NNTP distribution conf.h file and nntpxmit's get_tcp_conn.c
 * subroutines so that nntpxfer could be used on more systems.
 * Stan Barber, November 7, 1989 <sob@bcm.tmc.edu>
 *
 */

#ifndef TIMEOUT
#define TIMEOUT (30*60)
#endif

#define MAX_ARTICLES 32768

int server;			/* stream socket to the nntp server */
FILE * rd_fp, * wr_fp;
char * Pname;

#define	rindex	strrchr
#define	index	strchr

void
magic (hostname)
char	*hostname;
{
	char buf[BUFSIZ];

	if ((server = get_tcp_conn(hostname)) < 0) {
		perror("nntpxfer: could not open socket");
		exit(1);
	}
	if ((rd_fp = fdopen(server,"r")) == (FILE *) 0) {
		perror("nntpxfer: could not fdopen socket");
		exit(1);
	}

	if (optv) {
		printf("connected to nntp server at %s\n", hostname);
	}
	sockread(buf);
	if (optv) {
		printf("%s\n", buf);
	}
	if (buf[0] != '2') {	/* uh-oh, something's wrong! */
		printf("%s: protocol error: got '%s'\n", Pname,buf);
		close(server);
		exit(1);
	}


	/* first, tell them we're a slave process to get priority */
	sockwrite("SLAVE");
	sockread(buf);
	if (optv) {
		printf("%s\n", buf);
	}
	if (buf[0] != '2') {	/* uh-oh, something's wrong! */
		printf("%s: protocol error: got '%s'\n", Pname,buf);
		close(server);
		exit(1);
	}
}

static	jmp_buf	SFGstack;

static SIGRET
to_sfgets()
{
	longjmp(SFGstack, 1);
}

int
sockread(buf)
char *buf;
{
	int	esave, rz;
	char * ret;
	if (setjmp(SFGstack)) {
		alarm(0);	/* reset alarm clock */
		signal(SIGALRM, SIG_DFL);
		errno = ETIMEDOUT;		/* connection timed out */
		perror("nntpxfer: read error on server socket");
		close(server);
		exit(1);
	}
	signal(SIGALRM, to_sfgets);
	alarm(TIMEOUT);
	ret  = fgets(buf, BUFSIZ, rd_fp);
	esave = errno;
	alarm(0);			/* reset alarm clock */
	signal(SIGALRM, SIG_DFL);	/* reset SIGALRM */
	errno = esave;
	rz = strlen(buf);
	buf[rz-2] = '\0';
	if (ret  == (char * ) 0) {
		perror("nntpxfer: read error on server socket");
		fclose(rd_fp);
		exit(1);
	}
	return(0);
}

void
sockwrite(buf)
char *buf;
{
	register int sz;
	char buf2[BUFSIZ];
	if (optv > 1) {
		printf("SOCKWRITE >>> %s\n", buf);
	}
	strcpy(buf2,buf);
	strcat(buf2,"\r\n");
	sz = strlen(buf2);
	if (write(server,buf2,sz) != sz) {
		printf("nntpxfer: write error on server socket\n");
		close(server);
		exit(1);
	}
}

