
/*
 *	active.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1995 by Robert Krten, all rights reserved.
 *
 *	This module contains the active file handler
 *
 *	1995 12 22	R. Krten		created
 *	1996 01 06	R. Krten		made VFsys-style
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include "news.h"

#define	ActiveChunkSize		256				// size of each allocating request

extern	char	*progname;					/* main.c */
extern	char	*version;					/* version.c */

extern	int		opti;
extern	int		optv;						/* main.c */
extern	ConfigT	*Config_active;				/* main.c, configuration item */

/*
 *	One entry of our active file
*/

typedef struct	activeEntry
{
	char	*name;							// name of newsgroup
	U32		minart;							// minimum article #
	U32		maxart;							// maximum article #
	off_t	offset;							// offset into active file
	char	*restOfLine;					// rest of line, if any
}	ActiveT;

ActiveT		*active;						// the active in-core image
int			activeNumAllocated;				// how many allocated
int			activeNumUsed;					// of those, how many used

void
activeStarter ()
{
	readActive ();
	activeEnder ();
	readActive ();
}

/*
 *	readActive gets the elements of the active file into memory,
 *	and sorts them.  The trick for writing back into the active
 *	file quickly and easily is that we maintain the offset into
 *	the beginning of each line.
*/

void
readActive ()
{
	FILE	*fp;
	char	buf [BUFSIZ];
	char	tmpname [BUFSIZ];
	char	word [BUFSIZ];
	char	*ptr;
	U32		minart, maxart;
	int		lnum;
	off_t	off;

	active = NULL;
	activeNumAllocated = activeNumUsed = 0;
	fp = fopen (Config_active -> items [1].string, "r");
	if (fp != NULL) {
		off = 0;
		lnum = 0;
		while (!feof (fp) && fgets (buf, BUFSIZ, fp) != NULL) {
			buf [strlen (buf) - 1] = 0;
			lnum++;
			ptr = buf;
			if (!getWord (&ptr, tmpname)) {
				fprintf (stderr, "%s:  active file is missing the newsgroup name, %s:%d\n", progname, Config_active -> items [1].string, lnum);
			} else {
				if (!getWord (&ptr, word)) {
					fprintf (stderr, "%s:  active file is missing the maximum article number field, %s:%d\n", progname, Config_active -> items [1].string, lnum);
					maxart = 0;
				} else {
					maxart = atoi (word);
					if (!getWord (&ptr, word)) {
						minart = 0;
					} else {
						minart = atoi (word);
					}
				}
			}
			addActiveEntry (tmpname, maxart, minart, off, ptr);
			off = ftell (fp);
		}
		sortActive ();
		fclose (fp);
	}
}

void
activeEnder ()
{
	FILE	*fp;
	int		i;

	if ((fp = fopen (Config_active -> items [1].string, "w")) == NULL) {
		fprintf (stderr, "%s:  can't open active file %s for w!\n", progname, Config_active -> items [1].string);
		perror (NULL);
		exit (1);
	}

	for (i = 0; i < activeNumUsed; i++) {
		if (active [i].restOfLine) {
			fprintf (fp, "%s %010d %010d %s\n", active [i].name, active [i].maxart, active [i].minart, active [i].restOfLine);
		} else {
			fprintf (fp, "%s %010d %010d\n", active [i].name, active [i].maxart, active [i].minart);
		}
		free (active [i].name);
		if (active [i].restOfLine) {
			free (active [i].restOfLine);
		}
	}

	fclose (fp);

	active = NULL;
	activeNumUsed = activeNumAllocated = 0;
	free (active);
}

void
addNewActiveEntry (grp)
char	*grp;
{
	FILE	*fp;
	off_t	off;

	if ((fp = fopen (Config_active -> items [1].string, "a")) == NULL) {
		fprintf (stderr, "%s:  can't open active file %s for a!\n", progname, Config_active -> items [1].string);
		perror (NULL);
		exit (1);
	}

	off = ftell (fp);
	fprintf (fp, "%s %010d %010d y\n", grp, 0, 0);
	fclose (fp);
	addActiveEntry (grp, (U32) 0, (U32) 0, off, "y");
	sortActive ();
}

void
addActiveEntry (grp, mx, mn, off, rol)
char	*grp;
U32		mx;
U32		mn;
off_t	off;
char	*rol;
{
	if (activeNumAllocated == activeNumUsed) {
		activeNumAllocated += ActiveChunkSize;
		if (active == NULL) {
			active = calloc (ActiveChunkSize, sizeof (ActiveT));
			if (active == NULL) {
				fprintf (stderr, "%s:  failed calloc of %d bytes!\n", progname, sizeof (ActiveT) * activeNumAllocated);
				exit (1);
			}
		} else {
			active = realloc (active, sizeof (ActiveT) * activeNumAllocated);
			if (active == NULL) {
				fprintf (stderr, "%s:  failed realloc of %d bytes!\n", progname, sizeof (ActiveT) * activeNumAllocated);
				exit (1);
			}
		}
	}

	active [activeNumUsed].name = strdup (grp);
	active [activeNumUsed].maxart = mx;
	active [activeNumUsed].minart = mn;
	active [activeNumUsed].offset = off;
	if (rol != NULL) {
		active [activeNumUsed].restOfLine = strdup (rol);
	}
	activeNumUsed++;
}

//	the compare routine for the qsort
int
activeCompareQ (a, b)
ActiveT	*a, *b;
{
	return (strcmp (a -> name, b -> name));
}

void
sortActive ()
{
	qsort (active, activeNumUsed, sizeof (ActiveT), activeCompareQ);
}

//	the compare routine for the bsearch
int
activeCompareB (a, b)
char	*a;
ActiveT	*b;
{
	return (strcmp (a, b -> name));
}

ActiveT	*
getActive (grp)
char	*grp;
{
	return (bsearch (grp, active, activeNumUsed, sizeof (ActiveT), activeCompareB));
}

void
activeWriteGroup (grp)
char	*grp;
{
	ActiveT	*ptr;
	FILE	*fp;

	if ((ptr = getActive (grp)) == NULL) {
		if (optv) {
			printf ("%s:  hmmm... group %s isn't in the active file, but we were told to write it.\n", progname, grp);
		}
	} else {
		if ((fp = fopen (Config_active -> items [1].string, "r+")) == NULL) {
			fprintf (stderr, "%s:  can't updated active file for r+\n", progname);
			perror (NULL);
			return;		/* assume it's a temporary problem that might be fixed later */
		}
		fseek (fp, ptr -> offset, 0);
		fprintf (fp, "%s %010d %010d", grp, ptr -> maxart, ptr -> minart);
		fclose (fp);
	}
}

int
incrementAndGetArticleNumber (grp)
char	*grp;
{
	ActiveT	*ptr;

	if ((ptr = getActive (grp)) == NULL) {
		addNewActiveEntry (grp);
		if ((ptr = getActive (grp)) == NULL) {
			fprintf (stderr, "%s:  unable to add new group %s\n", progname, grp);
			return (0);
		}
	}
	ptr -> maxart++;
	activeWriteGroup (grp);
	return (ptr -> maxart);
}

