
/*
 *	filter.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1995 by Robert Krten, all rights reserved.
 *
 *	This module contains the filtration routines for determining
 *	if we do or do not accept an article / newsgroup / whatever.
 *
 *	1995 12 24	R. Krten		created
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include <regex.h>
#include "newslib.h"

#define	MaxRegexps		32

extern	char	*progname;					/* main.c */
extern	char	*version;					/* version.c */

extern	int		optv;						/* main.c */
extern	ConfigT	*Config_database;			/* main.c, location of database */

static	regex_t	regexps [MaxRegexps];
static	int		numRegexps;

static	regex_t	sanity [MaxRegexps];
static	int		numSanity;

void
filterStarter ()
{
	FILE	*fp;
	char	buf [BUFSIZ];

	numRegexps = 0;
	numSanity = 0;

	sprintf (buf, "%s/accept", Config_database -> items [1].string);
	if ((fp = fopen (buf, "r")) == NULL) {
		fprintf (stderr, "%s:  couldn't open %s for r, assuming you want all groups available!\n", progname, buf);
		// don't bother adding one, the for loop will just exit quietly
	} else {
		while (!feof (fp) && fgets (buf, BUFSIZ, fp) != NULL) {
			buf [strlen (buf) - 1] = 0;
			if (*buf && *buf != '#') {
				filterAddRegexp (regexps, &numRegexps, buf);
			}
		}
		fclose (fp);
	}

	/*
	 *	the sanity package
	 *	These regexp(s) have to match, otherwise the newsgroup is rejected
	*/

	//	fails any newsgroups that don't have "newsgroup[.newsgroup]..."
	filterAddRegexp (sanity, &numSanity, "^[^.]+(\.[^.]+)*$");
}

void
filterEnder ()
{
	int		i;

	for (i = 0; i < numRegexps; i++) {
		regfree (&regexps [i]);
	}
	numRegexps = 0;
}

void
filterAddRegexp (array, index, regexp)
regex_t	*array;
int		*index;
char	*regexp;
{
	int		err;
	char	ebuff [BUFSIZ];

	if (*index >= MaxRegexps) {
		fprintf (stderr, "%s:  too many regular expressions, limit %d\n", progname, MaxRegexps);
		exit (1);
	}

	if (err = regcomp (&array [*index], regexp, REG_NOSUB | REG_EXTENDED)) {
		fprintf (stderr, "%s:  couldn't add regular expression %s\n", progname, regexp);
		regerror (err, &array [*index], ebuff, BUFSIZ);
		fprintf (stderr, "%s:  %s\n", ebuff);
		exit (1);
	}
	++*index;
}

int
filterAcceptNewsgroup (group)
char	*group;
{
	int		i;

	/*
	 *	check against sanity package first
	 *
	 *	We apply negative logic here, because this is the list of
	 *	newsgroups that we *don't* want.
	*/

	for (i = 0; i < numSanity; i++) {
		if (regexec (&sanity [i], group, 0, NULL, 0)) {
			return (0);
		}
	}

	/*
	 *	now check against user-level regexps
	*/

	for (i = 0; i < numRegexps; i++) {
		if (!regexec (&regexps [i], group, 0, NULL, 0)) {
			return (1);
		}
	}
	return (0);
}

