
/*
 *	main.c
 *
 *	QNX 4, main.c shell version 0.004
 *
 *	(C) Copyright 1995, 1996 by Robert Krten, all rights reserved.
 *
 *	This module represents the main module for the NNTP KNews injection.
 *
 *	This program will fetch NNTP news and inject them into the KNews
 *	system.
 *
 *	1996 01 05	R. Krten		created
 *	1996 03 12	R. Krten		cleaned up for release
*/

#ifdef __USAGE

%C [options] arguments

where [options] are optional parameters chosen from:
    -v             verbose operation (more -v's == more verbose)
    -c cfgfile     override default configuration file /etc/knews
                   with "cfgfile"
    -d             debug mode -- don't delete temp files, implies -v
    -i             non-interactive mode -- don't prompt, just do it!
    -INIT          initialize entire database, erase everything!
    -a grp         add this group to VFsys.knews
    -e             do expiry
    -D             cause VFsys to write debug file
    -E exptime     force expiry to happen, as if time now was exptime

This program is the main interface into the KNews system.

#endif

#include "news.h"

static	void	optproc (int, char **);

char	*progname = "knews.nntp";
char	*blankname= "          ";

extern	char	*version;					/* version.c */
extern	char	*configFname;				/* library:  config.c */

ConfigT	*Config_newsfeed;					/* public -- type of newsfeed */
ConfigT	*Config_database;					/* public -- location of database */
ConfigT	*Config_active;						/* public -- location of active file */
ConfigT	*Config_expiry;						/* public -- expiration policy */
ConfigT	*Config_granularity;				/* public -- expiration granularity */

int		optd;
int		opte;
int		optE;
int		optI;
int		opti;
int		optv;

main (argc, argv)
int		argc;
char	**argv;
{
	optproc (argc, argv);

	if (optI) {
		starters ();
//		initializeDatabase ();
fprintf (stderr, "%s:  not supported\n", progname);
	} else if (opte || optE) {
		doExpire ();
	} else {
		starters ();
		getNews ();
	}
	enders ();
	return (0);
}

void
usageError ()
{
	fprintf (stderr, "%s:  error in use, type \"use %s\" for usage\n", progname, progname);
	exit (1);
}

void
optproc (argc, argv)
int		argc;
char	**argv;
{
	int		opt;
	int		didAnyOneshots;

	if (!argc) {
		usageError ();
	}

	optd = 0;
	opte = 0;
	optE = 0;
	optI = 0;
	didAnyOneshots = 0;

	while ((opt = getopt (argc, argv, "a:c:I:E:dDeiv")) != -1) {
		switch (opt) {
		case	'a':
			IPCAddNewsgroup (optarg);
			didAnyOneshots++;
			break;
		case	'c':
			configFname = optarg;
			break;
		case	'd':
			optd = 1;
			optv++;
			break;
		case	'D':
			IPCWriteDebug ();
			didAnyOneshots++;
			break;
		case	'e':
			opte = 1;
			break;
		case	'E':
			optE = atol (optarg);
			if (optv) {
				printf ("got optE %09ld\n", optE);
			}
			break;
		case	'I':
			if (strcmp (optarg, "NIT")) {
				fprintf (stderr, "%s:  bungled -INIT option with -I%s\n", progname, optarg);
				exit (1);
			}
			optI++;
			break;
		case	'i':
			opti = 1;
			break;
		case	'v':
			optv++;
			if (optv > 1) {
				printf ("Verbosity is now %d\n", optv);
			}
			break;
		default:
			usageError ();
			break;
		}
	}

	for (; optind < argc; optind++) {
		printf ("additional parameters [%d] -%s-\n", optind, argv [optind]);
	}

	if (didAnyOneshots) {
		exit (0);
	}

	getConfigInfo ();
}

void
getConfigInfo ()
{
	// newsfeed
	if ((Config_newsfeed = getConfig ("newsfeed")) == NULL) {
		fprintf (stderr, "%s:  no newsfeed specified in config file %s!\n", progname, configFname);
		exit (1);
	}
	if (strcmp (Config_newsfeed -> items [1].string, "nntp")) {
		fprintf (stderr, "%s:  didn't specify nntp in config file %s!\n", progname, configFname);
		exit (1);
	}
	if (Config_newsfeed -> nitems != 3) {
		fprintf (stderr, "%s:  wrong number of parameters specified for 'newsfeed' configuration item in file %s\n", progname, configFname);
		exit (1);
	}

	// database
	if ((Config_database = getConfig ("database")) == NULL) {
		fprintf (stderr, "%s:  no database directory defined in config file %s!\n", progname, configFname);
		exit (1);
	}
	if (Config_database -> nitems != 2) {
		fprintf (stderr, "%s:  wrong number of parameters specified for 'database' configuration item in file %s\n", progname, configFname);
		exit (1);
	}

	// active
	if ((Config_active = getConfig ("active")) == NULL) {
		fprintf (stderr, "%s:  no active filename defined in config file %s!\n", progname, configFname);
		exit (1);
	}
	if (Config_active -> nitems != 2) {
		fprintf (stderr, "%s:  wrong number of parameters specified for 'active' configuration item in file %s\n", progname, configFname);
		exit (1);
	}

	//	expiry
	if ((Config_expiry = getConfig ("expiry")) == NULL) {
		fprintf (stderr, "%s:  no expiry defined in config file %s!\n", progname, configFname);
		exit (1);
	}
	if (Config_expiry -> nitems != 2) {
		fprintf (stderr, "%s:  wrong number of parameters specified for 'expiry' configuration item in file %s\n", progname, configFname);
		exit (1);
	}
	if (Config_expiry -> items [1].number < 1) {
		fprintf (stderr, "%s:  invalid expiry, must be greater than zero in file %s!\n", progname, configFname);
		exit (1);
	}

	//	granularity
	if ((Config_granularity = getConfig ("granularity")) == NULL) {
		fprintf (stderr, "%s:  no granularity defined in config file %s!\n", progname, configFname);
		exit (1);
	}
	if (Config_granularity -> nitems != 2) {
		fprintf (stderr, "%s:  wrong number of parameters specified for 'granularity' configuration item in file %s\n", progname, configFname);
		exit (1);
	}
	if (Config_granularity -> items [1].number < 1) {
		fprintf (stderr, "%s:  invalid granularity, must be greater than zero in file %s!\n", progname, configFname);
		exit (1);
	}
}

void
starters ()
{
	filterStarter ();
	magic (Config_newsfeed -> items [2].string);
	activeStarter ();
}

void
enders ()
{
	filterEnder ();
	activeEnder ();
}

