
/*
 *	news.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1995 by Robert Krten, all rights reserved.
 *
 *	This module contains the NNTP RFC-977 newsreader.  It will read
 *	news from the NNTP feed and put it into the system.  It will also
 *	update VFsys.knews with any new files that it adds.
 *
 *	1995 12 31	R. Krten		created
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include "news.h"

#define	DAYS		(24 * HOURS)
#define	HOURS		(60 * MINUTES)
#define	MINUTES		60

extern	char	*progname;					/* main.c */
extern	char	*version;					/* version.c */

extern	int		optd;						/* main.c -- debug flag */
extern	int		optE;						/* expiration time */
extern	int		optv;						/* main.c */
extern	ConfigT	*Config_active;				/* main.c, location of active file */
extern	ConfigT	*Config_database;			/* main.c, location of database */
extern	ConfigT	*Config_expiry;				/* main.c, expiry policy, #days */
extern	ConfigT	*Config_granularity;		/* main.c, expiration granularity, #hours */
extern	LPTRT	*lptr;

static	FILE	*articleReaderFP;			/* contains article body */
static	FILE	*newsIDFP;					/* contains newnews output */
static	char	NewsIDFile [BUFSIZ];		// name of the file containing the ones to do
static	StringArrayT	*currentGroups;

extern	void getNewsReader (char *);
extern	void getNewsReaderSE (int);
extern	void getNewsHeadSE (int);
extern	void getNewsBodyReader (char *);
extern	void getNewsBodySE (int);

char	lastRunFilename [BUFSIZ];
time_t	lastRunTime;

void
getNewsStarter ()
{
	FILE	*fp;
	time_t	now;

	doExpire ();

	sprintf (lastRunFilename, "%s/lastruninfo", Config_database -> items [1].string);
	if ((fp = fopen (lastRunFilename, "r")) == NULL) {
		if (optv) {
			printf ("Couldn't open %s, using lastruntime as 0\n", lastRunFilename);
			perror (NULL);
		}
		lastRunTime = 0;
	} else {
		fscanf (fp, "%ld", &lastRunTime);
		fclose (fp);
	}

	if (optv) {
		printf ("\nLast run time was %d = %s", lastRunTime, ctime (&lastRunTime));
	}

	time (&now);

	if ((fp = fopen (lastRunFilename, "w")) == NULL) {
		if (optv) {
			printf ("Couldn't open %s for w, lost lastruntime\n", lastRunFilename);
			perror (NULL);
		}
	} else {
		fprintf (fp, "%ld\n", now);
		fclose (fp);
	}

	sprintf (NewsIDFile, "%s/news_ids", Config_database -> items [1].string);
}

void
getNewsEnder ()
{
}

void
doExpire ()
{
	LPTRT	*lp;
	time_t	expiry;
	time_t	now;
	int		i;
	char	fname [BUFSIZ];

	if (optE) {
		now = optE;
	} else {
		time (&now);
	}
	IPCExpire (now);
	if ((lp = readDir (Config_database -> items [1].string, "*.x")) != NULL) {
		for (i = 0; i < lp -> nlptrs; i++) {
			sscanf (lp -> object [i], "%d", &expiry);
			sprintf (fname, "%s/%s", Config_database -> items [1].string, lp -> object [i]);
			if (expiry < now) {
				if (optv > 1) {
					printf ("database file %s expired, removing\n", fname);
				}
				unlink (fname);
			}
		}
		lptr = lp;
		lptrInit ('e');
		free (lp);
	}
}

void
getNews ()
{
	FILE	*fpr;
	char	buf [BUFSIZ];
	char	fname [BUFSIZ];
	time_t	expiry;
	time_t	now;
	off_t	pos1, pos2;
	int		i, j, sts;
	long	startPos, endPos;
	struct	utimbuf utb;

	getNewsStarter ();

	/*
	 *	Contact the NNTP server to get the list of articles that have
	 *	arrived since our last run.  Note that if we are running
	 *	after having faulted previously, our temp file is still lying
	 *	around -- this is fine, we just append to it.
	*/

	nntp_NEWNEWS (lastRunTime, "*", getNewsReader, getNewsReaderSE);
	if ((fpr = fopen (NewsIDFile, "r+")) == NULL) {
		fprintf (stderr, "%s:  can't open %s, which I just created!\n", progname, NewsIDFile);
		perror (NULL);
		exit (1);
	}

	while (!feof (fpr)) {

		/*
		 *	look at the record we are reading.  Is it one that we've
		 *	already processed (because we are recovering a previous
		 *	work file)?
		*/

		pos1 = ftell (fpr);
		if (fgets (buf, BUFSIZ, fpr) == NULL) {
			break;
		}
		if (*buf == '+') {			// already been processed
			continue;
		}
		pos2 = ftell (fpr);

		/*
		 *	mark it as processed.
		*/

		*buf = '+';
		fseek (fpr, pos1, 0);		// get to beginning of line
		fwrite (buf, 1, 1, fpr);	// change the control character to a plus
		fflush (fpr);
		fseek (fpr, pos2, 0);		// get to last position
		buf [strlen (buf) - 1] = 0;

		/*
		 *	get a header for the article being processed
		*/

		nntp_HEAD (buf + 1, lptrReader, getNewsHeadSE);
		// we keep the LPTRS structure around!
		if (currentGroups == NULL) {
			lptrInit ('e');			// done with lptr structure
			continue;
		}

		/*
		 *	calculate the expiration time.  This is simply the
		 *	current time + a fixed number of days (from the config
		 *	file).  Convert the expiration time into a filename,
		 *	and append the article at the end of that file, noting
		 *	the position that the article is placed (ftell).
		*/

		time (&now);
		expiry = now + Config_expiry -> items [1].number * DAYS;
		expiry = expiry - (expiry % (Config_granularity -> items [1].number * HOURS));	// set granularity to 1 hour
		sprintf (fname, "%s/%09ld.x", Config_database -> items [1].string, expiry);
		if ((articleReaderFP = fopen (fname, "a")) == NULL) {
			fprintf (stderr, "%s:  can't open %s for a\n", progname, fname);
			perror (NULL);
			exit (1);
		}
		startPos = ftell (articleReaderFP);

		/*
		 *	write out the article.  First, write out the header,
		 *	which we already have in RAM in an lptr array
		*/

		lptrWriteFP (articleReaderFP);
		lptrInit ('e');					// done with lptr structure
		fputs ("\n", articleReaderFP);

		/*
		 *	next, call nntp_BODY to cause the article's body to
		 *	be passed through getNewsBodyReader, which just writes
		 *	it to the file.
		*/

		nntp_BODY (buf + 1, getNewsBodyReader, NULL);

		/*
		 *	capture the end position, so we can calculate the
		 *	number of bytes in the article.  Also, timestamp the
		 *	database file to show when it will expire (do an ls -l to
		 *	see this).
		*/

		fflush (articleReaderFP);
		endPos = ftell (articleReaderFP);
		fclose (articleReaderFP);
		utb.actime = utb.modtime = expiry;
		utime (fname, &utb);

		/*
		 *	for all valid newsgroups in the article's header, as
		 *	stashed away in the currentGroups member, add the article
		 *	to the active file, and to VFsys.knews.
		*/

		for (i = 0; i < currentGroups -> num; i++) {
			if (currentGroups -> ptrs [i][0]) {
				j = incrementAndGetArticleNumber (currentGroups -> ptrs [i]);
				if (optv) {
					printf ("Adding article #%d to newsgroup %s, fileID %09ld, startpos %ld endpos %ld (bytes %ld)\n", j, currentGroups -> ptrs [i], expiry, startPos, endPos, endPos - startPos);
				}
				if ((sts = IPCAddArticle (currentGroups -> ptrs [i], j, expiry, startPos, endPos - startPos)) != EOK) {
					fprintf (stderr, "%s:  couldn't add article, group %s articleID %s, return %d\n", progname, currentGroups -> ptrs [i], buf + 1, sts);
				}
			}
		}
		free (currentGroups);
	}
	fclose (fpr);

	if (!optd) {
		unlink (NewsIDFile);
	}

	getNewsEnder ();
}

StringArrayT *
crackNewsgroups (buf)
char	*buf;
{
	StringArrayT	*sa;
	int		i;
	int		less;

	if (optv > 1) {
		printf ("crackNewsgroups got \"%s\"\n", buf);
	}
	sa = stringToStringArray (buf, ",");
	less = 0;
	for (i = 0; i < sa -> num; i++) {
		if (!filterAcceptNewsgroup (sa -> ptrs [i])) {
			sa -> ptrs [i][0] = 255;
			sa -> ptrs [i][1] = 0;
			less++;
		}
	}
	sortStringArray (sa);
	sa -> num -= less;
	if (!sa -> num) {
		free (sa);
		return (NULL);
	}

	if (optv > 1) {
		for (i = 0; i < sa -> num; i++) {
			printf ("NEWSGROUP %d:  \"%s\"\n", i, sa -> ptrs [i]);
		}
	}
	return (sa);
}

void
analyzeHeaderLPTR ()
{
	int		i;
	char	*ptr, *optr;
	char	word [BUFSIZ];

	currentGroups = NULL;
	for (i = 0; i < lptr -> nlptrs; i++) {
		ptr = optr = lptr -> object [i];
		getWord (&ptr, word);
		if (optv > 1) {
			printf ("Word %s (%s)\n", word, optr);
		}
		if (!stricmp (word, "newsgroups:")) {
			getWord (&ptr, word);
			currentGroups = crackNewsgroups (word);
		}
	}
}

void
getNewsBodyReader (buf)
char	*buf;
{
	strcat (buf, "\n");
	fputs (buf, articleReaderFP);
}

void
getNewsHeadSE (type)
int		type;
{
	switch (type) {
	case	's':
		lptrInit ('s');
		break;
	case	'e':
		analyzeHeaderLPTR ();
		/* do not lptrInit ('e') here, we need it! */
		break;
	}
}

void
getNewsReader (buf)
char	*buf;
{
	char	sign;

	sign = '-';
	fwrite (&sign, 1, 1, newsIDFP);
	if (optv) {
		puts (buf);
	}
	strcat (buf, "\n");
	fputs (buf, newsIDFP);
}

void
getNewsReaderSE (type)
int		type;
{
	switch (type) {
	case	's':
		if ((newsIDFP = fopen (NewsIDFile, "r+")) != NULL) {
			printf ("%s:  found a previous temp file, recovering...\n", progname);
			fseek (newsIDFP, 0, 2);		// go to end, append
		} else {						// no temp file, create one
			if ((newsIDFP = fopen (NewsIDFile, "w+")) == NULL) {
				fprintf (stderr, "%s:  can't open %s for w\n", progname, NewsIDFile);
				perror (NULL);
				exit (1);
			}
		}
		break;
	case	'e':
		fclose (newsIDFP);
		newsIDFP = NULL;
		break;
	}
}

