
/*
 *	MAIN.C
 *
 *	QNX 4, MAIN.C shell version 0.004
 *
 *	(C) Copyright 1992 by Robert Krten, all rights reserved.
 *
 *	This module represents the main module for the build manager utility
 *
 *	1992 04 18	R. Krten		created
 *	1993 12 17	R. Krten		fixed up for USAGE and more options
*/

#ifdef __USAGE

%C [options] [versionFile]

where [options] are optional parameters chosen from:
    -h             suppress "Reason" query
    -t             suppress update of version file
    f=filename     specify log file [default build.log]

By default, BM updates the versionFile to the next version, and prompts
for a revision history line which it then appends to build.log.

#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

static	void	optproc (int, char **);
static	void	usage (int);
extern	FILE	*efopen (char *, char *);

char	infile [128];
char    outfile [128];
FILE    *fpo;

int		opth;
int		optt;

char	*progname = "bm";
char	*blankname= "  ";
extern	char *version;					/* version.c */

main (argc, argv)
int     argc;
char    **argv;
{
	optproc (argc, argv);
	fprintf (stderr, "Build Manager Utility, Version %s\n", version);
	bm (infile);
	fclose (fpo);
	exit (0);
}

void
usage (n)
int		n;
{
	fprintf (stderr, "%s:  error in use, type \"use %s\" for usage\n", progname, progname);
	if (n) {
		exit (n);
	}
}

bm (fn)
char    *fn;
{
	int     i;
	int     pb;
	FILE    *fp;
	int     updated;
	char    buf [512];
	char    pbuf [512];
	struct  tm  *t;
	time_t	tval;

	if (optt) {
		fp = efopen (fn, "r");
		fgets (buf, 512, fp);
		buf [strlen (buf) - 1] = 0;
		fclose (fp);
		pb = 0;
		for (i = 0; i < strlen (buf); i++) {
			if (buf [i] >= '0' && buf [i] <= '9') {
				pbuf [pb++] = buf [i];
			}
		}
		pbuf [pb] = 0;
		printf ("Version %s ", pbuf);
		updated = 0;
		for (i = strlen (buf) - 1; i >= 0 && !updated; i--) {
			if (buf [i] >= '0' && buf [i] < '9') {
				buf [i]++;
				updated = 1;
			} else if (buf [i] == '9') buf [i] = '0';
		}
		if (!updated) {
			printf ("could not be updated.\n");
			exit (1);
		}
		fp = efopen (fn, "w");
		fprintf (fp, "%s\n", buf);
		fclose (fp);
		pb = 0;
		for (i = 0; i < strlen (buf); i++) {
			if (buf [i] >= '0' && buf [i] <= '9') {
				pbuf [pb++] = buf [i];
			}
		}
		pbuf [pb] = 0;
		printf ("updated to version %s\n", pbuf);
	} else {
		*pbuf = 0;
	}
	if (opth) {
		fpo = efopen (outfile, "a");
		printf ("Reason:  ");
		fgets (buf, 512, stdin);
		time (&tval);
		t = localtime (&tval);
		fprintf (fpo, "%02d%02d%02d %s:  %s", t -> tm_year, t -> tm_mon + 1, t -> tm_mday, pbuf, buf);
	}
}

void
optproc (argc, argv)
int		argc;
char	**argv;
{
	int		i;
	int		optFound;

	if (!argc) {
		usage (1);
	}

	optt = 1;
	opth = 1;
	*outfile = 0;
	*infile = 0;

	if (argc > 1) {
		for (i = 1; i < argc; i++) {
			optFound = 0;
			if (argv [i][0] == '-') {
				optFound = '-';
			} else if (argv [i][0] == '+') {
				optFound = '+';
			}
			if (optFound) {
				optFound = (optFound == '+');
				switch (argv [i][1]) {
				case	'h':
					opth = optFound;
					break;
				case	't':
					optt = optFound;
					break;
				default:
					usage (1);
					break;
				}
			} else {
				if (argv [i][1] == '=') {
					switch (argv [i][0]) {
					case	'f':
						if (*outfile) {
							usage (1);
						} else {
							strcpy (outfile, &argv [i][2]);
						}
						break;
					default:
						usage (1);
						break;
					}
				} else {
					if (*infile) {
						usage (1);
					} else {
						strcpy (infile, argv [i]);
					}
				}
			}
		}
	}

	if (optt && !*infile) {
		usage (1);
	}
	if (!*outfile) {
		strcpy (outfile, "build.log");
	}
}

FILE *
efopen (name, mode)
char	*name;
char	*mode;
{
	FILE	*fp;

	if ((fp = fopen (name, mode)) == NULL) {
		fprintf (stderr, "%s:  can't open %s for %s\n", progname, name, mode);
		perror (NULL);
		exit (1);
	}
	return (fp);
}

