
/*
 *	knews.h
 *
 *	QNX 4
 *
 *	(C) Copyright 1996 by Robert Krten, all rights reserved.
 *
 *	This module contains the manifest constants and declarations for the
 *	VFsys.knews resource manager.
 *
 *	1996 01 04	R. Krten		created
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <time.h>
#include <dirent.h>
#include <fcntl.h>
#include <ctype.h>
#include <utime.h>
#include <pwd.h>

#include <sys/kernel.h>
#include <sys/osinfo.h>
#include <sys/fd.h>
#include <sys/psinfo.h>
#include <sys/sched.h>
#include <sys/prfx.h>
#include <sys/stat.h>
#include <sys/name.h>
#include <sys/types.h>

#include "../lib/portable.h"

#include "database.h"
#include "resmgr.h"

#include "../lib/ipc.h"
#include "../lib/config.h"
#include "../lib/lptr.h"
#include "../lib/readdir.h"
#include "../lib/utils.h"

/*
 *	Prototypes
*/

//	db_fsys.c

void			databaseStarter ();
void			databaseEnder ();
void			freeDirNode (DirNodeT *node);
void			freeFileNode (FileNodeT *node);
DirNodeT *		cdOneLevel (DirNodeT *node, char *path);
DirNodeT *		cdMultiLevel (char *path);
DirNodeT *		mdOneLevel (DirNodeT *node, char *path);
DirNodeT *		mdMultiLevel (char *);
int				rmLastLevel (char *);
FileNodeT *		getFileInfo (char *);
FileNodeT *		putFileInfo (char *, U32, U32, U32, U32);
FileNodeT *		findFileNode (DirNodeT *, U32);
DirNodeT *		allocDirNode (char *);
FileNodeT *		allocFileNode (void);
void			debugDumpDatabase (void);
void			databaseExpire (time_t);

//	utils.c

int				map_ocb (pid_t, int, OCBT *);
OCBT *			get_ocb (pid_t, int);
void			free_ocb_entry (pid_t, int);
void			dumpMsg_io_open (struct _io_open *);
void			buildStat (struct stat *, int, size_t);
void			utilsStarter (void);
void			utilsEnder (void);

//	resmgr.c
void			serve (void);
void			resmgrInit (void);

//	journal.c

void			journalStarter (void);
void			journalEnder (void);
void			writeJournal (int, char *);
void			journalExpire (time_t);
void			putGroupInfo (char *);

