
/*
 *	main.c
 *
 *	QNX 4, main.c shell version 0.004
 *
 *	(C) Copyright 1995, 1996 by Robert Krten, all rights reserved.
 *
 *	This module represents the main module for the VFsys.knews server.
 *
 *	This program will accept messages from clients, and perform
 *	services.  These services include:
 *
 *		administration of news directories
 *		access to news directories
 *
 *	1995 12 28	R. Krten		created
 *	1995 12 30	R. Krten		defined operation
 *	1996 01 05	R. Krten		moved to VFsys-style
 *	1996 03 12	R. Krten		cleaned up for release
*/

#ifdef __USAGE

%C [options] arguments

where [options] are optional parameters chosen from:
    -c cfgfile     override default configuration file /etc/knews
                   with "cfgfile"
    -D             debug mode
    -p prio        run with this priority (default 15)
    -v             verbose operation (can be repeated, default off)

VFsys.knews implements the local news filesystem.  It operates
by taking over a prefix space as defined in the configuration file,
and then manifesting the news hierarchy under that prefix space.

It operates on IPC messages, and depends upon various client programs
to tell it to do things -- it doesn't do anything on its own, except
manage the prefix space.

#endif

#include "knews.h"

static	void	optproc (int, char **);

char	*progname = "VFsys.knews";
char	*blankname= "           ";
extern	char	*version;					/* version.c */
extern	char	*configFname;				/* library:  config.c */

ConfigT	*Config_database;					/* public -- location of database */
char	*spooldir;							/* public -- name of directory to take over */

int		optD;
int		optp;
int		optv;

main (argc, argv)
int		argc;
char	**argv;
{
	printf ("The VFsys.knews virtual filesystem manager, version F%s\n", version);
	printf ("(C) Copyright 1995/1996 by PARSE Software Devices, all rights reserved.\n");

	optproc (argc, argv);

	printf ("Starting server..."); fflush (stdout);
    startup ();

    serve ();

	shutdown ();

	return (0);
}

void
usageError ()
{
	fprintf (stderr, "%s:  error in use, type \"use %s\" for usage\n", progname, progname);
	exit (1);
}

void
optproc (argc, argv)
int		argc;
char	**argv;
{
	int		opt;

	if (!argc) {
		usageError ();
	}

	optp = 15;		/* default priority */

	while ((opt = getopt (argc, argv, "c:p:vD")) != -1) {
		switch (opt) {
		case	'D':
			optD = 1;
			break;
		case	'c':
			configFname = optarg;
			break;
		case	'p':
			optp = atoi (optarg);
			if (optp < 4 || optp > 31) {
				fprintf (stderr, "%s:  illegal priority %d, range 4-31\n", progname, optp);
				exit (1);
			}
			break;
		case	'v':
			optv++;
			if (optv > 1) {
				printf ("Verbosity is now %d\n", optv);
			}
			break;
		default:
			usageError ();
			break;
		}
	}

	for (; optind < argc; optind++) {
		printf ("additional parameters [%d] -%s-\n", optind, argv [optind]);
	}
}

void
getConfigInfo ()
{
	ConfigT	*Config_newsdir;		/* local, copied to "spooldir" */

	Config_database = getConfig ("database");
	if (Config_database == NULL) {
		fprintf (stderr, "%s:  no database directory defined in config file %s!\n", progname, configFname);
		exit (1);
	}
	if (Config_database -> nitems != 2) {
		fprintf (stderr, "%s:  wrong number of parameters specified for 'database' configuration item in file %s\n", progname, configFname);
		exit (1);
	}

	Config_newsdir = getConfig ("news");
	if (Config_newsdir == NULL) {
		spooldir = "/usr/spool/news";
	} else {
		if (Config_newsdir -> nitems != 2) {
			fprintf (stderr, "%s:  wrong number of parameters specified for 'news' configuration item in file %s\n", progname, configFname);
			exit (1);
		}
		spooldir = Config_newsdir -> items [1].string;
	}

	if (optv) {
		printf ("%s:  using spool directory %s\n", progname, spooldir);
	}
}

void
startup ()
{
	getConfigInfo ();
	databaseStarter ();
	journalStarter ();
	utilsStarter ();
	resmgrInit ();
}

void
shutdown ()
{
	databaseEnder ();
	journalEnder ();
	utilsEnder ();
}

