
/*
 *	rm_io.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1995 by Robert Krten, all rights reserved.
 *
 *	This module contains the resource manager sections that deal
 *	with the IO processing.
 *
 *	1995 12 28	R. Krten		created
 *	1996 01 05	R. Krten		updated for VFsys-style
 *	1996 01 08	R. Krten		added IPC debug message
 *	1996 02 18	R. Krten		added lseek/fstat
 *	1996 03 09	R. Krten		removed from resmgr.c
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include "knews.h"

extern	char	*progname;					/* main.c */
extern	char	*version;					/* version.c */

extern	int		optv;						/* main.c, verbosity */
extern	ConfigT	*Config_database;			/* main.c, configuration item */

extern	MsgT	msg;						/* resmgr.c, global */

/*
 *	Directory handling resource messages
*/

int
io_chdir (pid)
pid_t	pid;
{
pid=pid;
	if (optv) {
		printf ("CHDIR_MSG:\n");
	}

	msg.chdir_reply.status = EOK;

	if (msg.chdir.path [0]) {				/* going somewhere */
		if (cdMultiLevel (msg.chdir.path) == NULL) {
			msg.chdir_reply.status = ENOENT;
		}
	}

	return (sizeof (msg.chdir_reply));
}

int
io_handle (pid)
pid_t	pid;
{
	static char *handleMsgs [7] =
	{
		"<bogus>",			"_IO_HNDL_INFO",	"_IO_HNDL_RDDIR",
		"_IO_HNDL_CHANGE",	"_IO_HNDL_UTIME",	"_IO_HNDL_LOAD",
		"_IO_HNDL_CLOAD"
	};

	OCBT	*ocb;

	msg.status = EOK;

	switch (msg.open.oflag) {
	case _IO_HNDL_CHANGE:
	case _IO_HNDL_UTIME:
	case _IO_HNDL_LOAD:
	case _IO_HNDL_CLOAD:
	default:
		if (optv) {
			printf ("%s:  unsupported\n", handleMsgs [msg.open.oflag]);
		}
		msg.status = ENOTSUP;
		break;

	case _IO_HNDL_INFO:
	case _IO_HNDL_RDDIR:
		if ((ocb = calloc (1, sizeof (OCBT))) == NULL) {
			msg.status = ENOMEM;
			break;
		}
		if (map_ocb (pid, msg.open.fd, ocb)) {
			msg.open_reply.status = errno;
			free (ocb);
			break;
		}
		if ((ocb -> cwd = cdMultiLevel (msg.open.path)) == NULL) {
			msg.open_reply.status = ENOENT;
			free (ocb);
			free_ocb_entry (pid, msg.open.fd);
			break;
		}
		ocb -> state = OCBState_Initial;
		ocb -> fd = -1;
		if (optv) {
			printf ("%s:  path %s got OCB %p cwd %p\n", handleMsgs [msg.open.oflag], msg.open.path, ocb, ocb -> cwd);
		}
		break;
	}
	return (sizeof (msg.open_reply));
}

int
io_readdir (pid)
pid_t	pid;
{
	struct	dirent	d;
	int		ndone;
	OCBT	*ocb;

	if (optv) {
		printf ("_IO_READDIR:\n");
		printf ("  type     %d (0x%x)\n", msg.readdir.type, msg.readdir.type);
		printf ("  fd       %d\n", msg.readdir.fd);
		printf ("  ndirs    %d\n", msg.readdir.ndirs);
	}

	if ((ocb = get_ocb (pid, msg.readdir.fd)) == NULL) {
		msg.readdir_reply.status = EBADF;
		return (sizeof (msg.readdir_reply));
	}

	if (optv > 1) {
		printf ("  ocb %p, cwd %p (%s)\n", ocb, ocb -> cwd, ocb -> cwd ? ocb -> cwd -> name : "<../NULL>");
	}

	ndone = 0;

	d.d_stat.st_status = 0;						// ensure d_stat not valid

	while (ndone < 8 && ocb -> state != OCBState_Done) {
		switch (ocb -> state) {
		case	OCBState_Initial:
			strcpy (d.d_name, ".");
			memcpy (msg.readdir_reply.data, &d, sizeof (struct dirent));
			strcpy (d.d_name, "..");
			memcpy (msg.readdir_reply.data + sizeof (struct dirent), &d, sizeof (struct dirent));
			ndone = 2;
			if (ocb -> cwd == NULL) {
				ocb -> state = OCBState_Done;
			} else {
				if (ocb -> cwd -> files) {		// any files to do?
					ocb -> cwf = ocb -> cwd -> files;
					ocb -> state = OCBState_GetFiles;
				} else {						// no files, do dirs
					if (ocb -> cwd -> down) {	// any dirs, then?
						ocb -> cwd = ocb -> cwd -> down;
						ocb -> state = OCBState_GetDirectories;
					} else {					// no dirs either, end
						ocb -> state = OCBState_Done;
					}
				}
			}
			break;
	
		case	OCBState_GetFiles:
			sprintf (d.d_name, "%d", ocb -> cwf -> artnum);
			memcpy (msg.readdir_reply.data + sizeof (struct dirent) * ndone, &d, sizeof (struct dirent));
			if (optv > 1) {
				printf ("_IO_READDIR (file) \"%s\"\n", d.d_name);
			}
			ndone++;
			if (ocb -> cwf -> next == NULL) {			// files done,
				if (ocb -> cwd -> down) {				// dirs?
					ocb -> cwd = ocb -> cwd -> down;
					ocb -> state = OCBState_GetDirectories;
				} else {
					ocb -> state = OCBState_Done;
				}
			} else {
				ocb -> cwf = ocb -> cwf -> next;
			}
			break;

		case	OCBState_GetDirectories:
			strcpy (d.d_name, ocb -> cwd -> name);
			memcpy (msg.readdir_reply.data + sizeof (struct dirent) * ndone, &d, sizeof (struct dirent));
			if (optv > 1) {
				printf ("_IO_READDIR (dir) \"%s\"\n", ocb -> cwd -> name);
			}
			ndone++;
			if (ocb -> cwd -> across == NULL) {		/* no more */
				ocb -> state = OCBState_Done;
			} else {
				ocb -> cwd = ocb -> cwd -> across;
			}
			break;
		}
	}

	if (optv > 1) {
		printf ("_IO_READDIR ndone %d\n", ndone);
	}

	msg.readdir_reply.ndirs = ndone;
	msg.readdir_reply.status = EOK;
	return (sizeof (msg.readdir_reply) + sizeof (struct dirent) * ndone + 1);
}

int
io_rewinddir (pid)
pid_t	pid;
{
	OCBT	*ocb;

	if (optv) {
		printf ("_IO_rewinddir:\n");
	}

	if ((ocb = get_ocb (pid, msg.rewinddir.fd)) == NULL) {
		msg.rewinddir_reply.status = EBADF;
		return (sizeof (msg.rewinddir_reply));
	}

	// reset directory information
	ocb -> state = OCBState_Initial;
	ocb -> fd = -1;

	msg.rewinddir_reply.status = EOK;
	return (sizeof (msg.rewinddir_reply));
}

int
io_fstat (pid)
pid_t	pid;
{
	OCBT	*ocb;

	if (optv) {
		printf ("_IO_FSTAT:\n");
	}

	if ((ocb = get_ocb (pid, msg.fstat.fd)) == NULL) {
		msg.fstat_reply.status = EBADF;
		return (sizeof (msg.fstat_reply));
	}
	if (ocb -> cwf != NULL) {
		buildStat (&msg.fstat_reply.stat, S_IFREG | 0444, ocb -> cwf -> length);
	} else {
		if (ocb -> cwd != NULL) {
			buildStat (&msg.fstat_reply.stat, S_IFDIR | 0555, 2048);
		} else {
			msg.fstat_reply.status = ENOSYS;		// errno correct???
		}
	}

	msg.fstat_reply.status = EOK;
	return (sizeof (msg.fstat_reply));
}

int
io_stat (pid)
pid_t	pid;
{
	FileNodeT		*fnode;
	
pid=pid;
	if (optv) {
		printf ("IO_STAT:\n");
		dumpMsg_io_open (&msg.open);
	}

	msg.fstat_reply.status = EOK;
	if (!msg.open.path [0]) {
		buildStat (&msg.fstat_reply.stat, S_IFDIR | 0555, 2048);	/* root dir is a dir */
	} else {
		if (cdMultiLevel (msg.open.path) != NULL) {
			if (optv) printf ("  cd worked (%s)\n", msg.open.path);
			buildStat (&msg.fstat_reply.stat, S_IFDIR | 0555, 2048);
		} else if ((fnode = getFileInfo (msg.open.path)) != NULL) {
			if (optv) printf ("  got fnode %p (%s)\n", fnode, msg.open.path);
			buildStat (&msg.fstat_reply.stat, S_IFREG | 0444, fnode -> length);
		} else {
			if (optv) printf ("  nothing worked (%s)\n", msg.open.path);
			msg.fstat_reply.status = ENOENT;		/* else unknown */
		}
	}

	return (sizeof (msg.fstat_reply));
}

int
io_open (pid)
pid_t	pid;
{
	char	buf [BUFSIZ];
	OCBT	*ocb;

	if (optv) {
		printf ("_IO_OPEN:\n");
	}

	msg.open_reply.status = EOK;

	if ((ocb = calloc (1, sizeof (OCBT))) == NULL) {
		msg.open_reply.status = ENOMEM;
		return (sizeof (msg.open_reply));
	}

	ocb -> state = OCBState_Initial;
	ocb -> fd = -1;

	if (!msg.open.path [0]) {
		msg.open_reply.status = EISDIR;
	} else {
		if ((ocb -> cwf = getFileInfo (msg.open.path)) == NULL) {
			msg.open_reply.status = ENOENT;
		} else {
			sprintf (buf, "%s/%09ld.x", Config_database -> items [1].string, ocb -> cwf -> expiry);
			if ((ocb -> fd = open (buf, O_RDONLY)) == -1) {
				msg.open_reply.status = errno;
			} else {
				lseek (ocb -> fd, ocb -> cwf -> offset, SEEK_SET);
				ocb -> position = 0;
			}
		}
	}

	if (msg.open_reply.status != EOK) {
		free (ocb);
	} else {
		if (map_ocb (pid, msg.open.fd, ocb)) {
			msg.open_reply.status = errno;
			free (ocb);
		}
	}

	return (sizeof (msg.open_reply));
}

int
io_close (pid)
pid_t	pid;
{
	OCBT	*ocb;

	if (optv) {
		printf ("_IO_CLOSE:\n");
	}

	if ((ocb = get_ocb (pid, msg.close.fd)) == NULL) {
		msg.close_reply.status = EBADF;
		return (sizeof (msg.close_reply));
	}

	if (ocb -> fd != -1) {
		close (ocb -> fd);
	}

	if (optv) {
		printf ("Freeing OCB %p\n", ocb);
		if (ocb -> fd != -1) {
			printf ("  closed fd %d\n", ocb -> fd);
		}
	}

	free (ocb);
	free_ocb_entry (pid, msg.close.fd);

	msg.close_reply.status = EOK;
	return (sizeof (msg.close_reply));
}

int
io_lseek (pid)
pid_t	pid;
{
	OCBT	*ocb;
	int		newpos;

	if (optv) {
		printf ("_IO_LSEEK:  whence %d offset %d\n", msg.lseek.whence, msg.lseek.offset);
	}

	if ((ocb = get_ocb (pid, msg.lseek.fd)) == NULL) {
		msg.lseek_reply.status = EBADF;
		return (sizeof (msg.lseek_reply));
	}

	msg.lseek_reply.status = EOK;
	switch (msg.lseek.whence) {
	case SEEK_END:
		newpos = ocb -> cwf -> length + msg.lseek.offset;
		break;
	case SEEK_SET:
		newpos = msg.lseek.offset;
		break;
	case SEEK_CUR:
		newpos = ocb -> position + msg.lseek.offset;
		break;
	default:
		msg.lseek_reply.status = EINVAL;
	}

	// since we are a read-only filesystem, we don't accept seeks past
	// the end of the buffer

	if (newpos > ocb -> cwf -> length) {
		msg.lseek_reply.status = EINVAL;
	}
	if (newpos < 0) {
		msg.lseek_reply.status = EINVAL;
	}
	if (msg.lseek_reply.status == EOK) {
		ocb -> position = newpos;
		msg.lseek_reply.offset = newpos;
	}

	lseek (ocb -> fd, ocb -> cwf -> offset + ocb -> position, SEEK_SET);
	return (sizeof (msg.lseek_reply));
}

int
io_read (pid)
pid_t	pid;
{
	OCBT	*ocb;
	int		num;
	int		len;

	if (optv) {
		printf ("_IO_READ:\n");
	}

	if ((ocb = get_ocb (pid, msg.read.fd)) == NULL) {
		msg.read_reply.status = EBADF;
		return (sizeof (msg.read_reply));
	}

	num = min (msg.read.nbytes, MAXMSGSIZE - sizeof (msg.read_reply) - 1);

	if (ocb -> state == OCBState_Initial) {
		len = ocb -> cwf -> length - ocb -> position;
		if (len <= num) {		/* less or equal left than requested */
			read (ocb -> fd, msg.read_reply.data, len);
			msg.read_reply.nbytes = len;
			ocb -> position = ocb -> cwf -> length;	/* set to EOF */
			ocb -> state = OCBState_Done;
		} else {									/* more left, multiple reads */
			read (ocb -> fd, msg.read_reply.data, num);
			msg.read_reply.nbytes = num;
			ocb -> position += num;
		}
	} else {
		msg.read_reply.nbytes = 0;
	}

	msg.read_reply.status = EOK;

	if (optv) {
		printf ("OCB %p nbytes %d msg\n", ocb, msg.read_reply.nbytes);
	}

	return (sizeof (msg.read_reply) + msg.read_reply.nbytes - 1);
}

int
io_bogus (pid)
pid_t	pid;
{
	fprintf (stderr, "%s:  io_bogus encountered!  pid %d sent it!\n", progname, pid);
	return (0);								/* don't reply, so we can hold it! */
}

int
io_nosys (pid)
pid_t	pid;
{
	pid=pid;		// keep compiler happy

	msg.status = ENOSYS;
	return (sizeof (msg.open_reply));
}

