
/*
 *	utils.c
 *
 *	QNX 4
 *
 *	(C) Copyright 1995 by Robert Krten, all rights reserved.
 *
 *	This module contains utilities that are bound to the KNews
 *	system.
 *
 *	1995 12 28	R. Krten		created
 *	1996 03 12	R. Krten		cleaned up for release
*/

#include "knews.h"

#define	MaxOCBs			0x0100
#define	ValidOCBMask	0x1000				// must be set, else invalid

extern	char	*progname;					/* main.c */
extern	char	*version;					/* version.c */

extern	int		optv;						/* main.c */
extern	ResmgrT	Resmgr;						/* resmgr.c, resmgr stuff */

static	int		uidNews, gidNews;
static	OCBT	*ocbs [MaxOCBs];
static	int		ocbsUsed [MaxOCBs];

void
utilsStarter ()
{
	struct	passwd	*uinfo;
	int		i;

	if ((uinfo = getpwnam ("news")) == NULL) {
		gidNews = 0;						/* use root */
		uidNews = 0;						/* use root */
	} else {
		gidNews = uinfo -> pw_gid;
		uidNews = uinfo -> pw_uid;
	}

	if (optv) {
		printf ("News is %d:%d\n", gidNews, uidNews);
	}

	for (i = 0; i < MaxOCBs; i++) {
		ocbs [i] = NULL;
		ocbsUsed [i] = 0;
	}
}

void
utilsEnder ()
{
}

/*
 *	OCB functions
 *
 *	map_ocb
 *	get_ocb
 *
 *	The Open Control Block (OCB) is the main "context" structure that
 *	is maintained between an "open" or "handle" message and the subsequent
 *	data messages.
 *
 *	QSSL doesn't really document this, except to say that "it digs
 *	deep into Proc to get information".  :-)
 *
 *	The value that can be stored with the qnx_fd_attach call is only
 *	24 bits long, whereas a pointer under the 32-bit model is, of
 *	course, 32-bits long.  Luckily, the top 8 bits of the pointer will
 *	MOST LIKELY be zero anyway, unless we exceed 16 MBYTES of allocated
 *	memory (possible).
 *
 *	This new revision maintains a separate array, and the OCB that gets
 *	put into the 24-bit area is only an index number into the array;
 *	the actual OCB pointer is stored in the array.
 *	The array is 256 entries, which is deemed to be sufficient.  If
 *	it fills up, VFsys.knews will issue a report to stderr.
*/

int
map_ocb (pid, fd, ocb)
pid_t	pid;
int		fd;
OCBT	*ocb;
{
	int		i;

	/*
	 *	Find an empty OCB pointer index
	*/

	for (i = 0; i < MaxOCBs; i++) {
		if (!ocbsUsed [i]) {
			break;
		}
	}

	if (i == MaxOCBs) {
		fprintf (stderr, "%s:  RAN OUT OF OCB ENTRIES!\n", progname);
		exit (1);
	}

	/*
	 *	Use unit as index into ocb array
	*/

	ocbsUsed [i] = 1;
	ocbs [i] = ocb;

	if (qnx_fd_attach (pid, fd, 0, 0, 0, 0, i | ValidOCBMask) == -1) {
		return (errno);
	}
	return (0);
}

OCBT *
get_ocb (pid, fd)
pid_t	pid;
int		fd;
{
	OCBT	*ocb;
	int		i;
   
	i = __get_fd (pid, fd, Resmgr.fd_ctrl);
	if (i == -1 || !(i & ValidOCBMask)) {
		return (NULL);
	}
	ocb = ocbs [i & ~ValidOCBMask];
	return (ocb);
}

void
free_ocb_entry (pid, fd)
pid_t	pid;
int		fd;
{
	int		i;

	i = __get_fd (pid, fd, Resmgr.fd_ctrl);
	if (i == -1 || !(i & ValidOCBMask)) {
		return;
	}
	i &= ~ValidOCBMask;
	ocbs [i] = NULL;
	ocbsUsed [i] = 0;
}

void
dumpMsg_io_open (iom)
struct	_io_open	*iom;
{
	printf ("\ttype          0x%04X\n", iom -> type);
	printf ("\tfd            %d\n", iom -> fd);
	printf ("\tunit          %d\n", iom -> unit);
	printf ("\tversion_cycle %d\n", iom -> version_cycle);
	printf ("\teflag         0x%0X\n", iom -> eflag);
	printf ("\tnid           %d\n", iom -> nid);
	printf ("\tpid           %d\n", iom -> pid);
	printf ("\tsflag         0x%04X\n", iom -> sflag);
	printf ("\toflag         0x%04X\n", iom -> oflag);
	printf ("\tmode          0x%04X\n", iom -> mode);
	printf ("\tzero2         %d\n", iom -> zero2);
	printf ("\tpath [0]      %c (%d)\n", iom -> path [0], iom -> path [0]);
	if (iom -> path [0]) {
		printf ("\tpath          \"%s\"\n", iom -> path);
	}
}

void
buildStat (statBuf, mode, size)
struct	stat	*statBuf;
int		mode;
size_t	size;
{
	time_t	currentTime;
	static dev_t inode = 1;

	time (&currentTime);

	statBuf -> st_ino = inode++;
	statBuf -> st_rdev = 54321;
//	statBuf -> st_rdev = (dev_t) ((getnid() << 16L) + (Resmgr.devno << 10L)); /* Major and Minor Device number w/node ID. */
	statBuf -> st_mode = mode;
	statBuf -> st_nlink = 2;
	statBuf -> st_ftime = currentTime;
	statBuf -> st_atime = currentTime;
	statBuf -> st_mtime = currentTime;
	statBuf -> st_ctime = currentTime;
	statBuf -> st_ouid = uidNews;
	statBuf -> st_ogid = gidNews;
	statBuf -> st_status = _FILE_USED;
	statBuf -> st_size = size;
}

